package com.example.bradysdk.printercommunication;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import android.Manifest;
import android.content.Context;

import androidx.test.rule.GrantPermissionRule;

import com.bradysdk.api.printerconnection.PrinterDetails;
import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

public class BluetoothPrinterConnectionTests implements PrinterDiscoveryListener {

    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation = GrantPermissionRule.grant(android.Manifest.permission.ACCESS_FINE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation2 = GrantPermissionRule.grant(Manifest.permission.ACCESS_COARSE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation3 = GrantPermissionRule.grant(Manifest.permission.ACCESS_BACKGROUND_LOCATION);

    @Rule public GrantPermissionRule mRuntimePermissionRuleBluetooth = GrantPermissionRule.grant(android.Manifest.permission.BLUETOOTH);
    @Rule public GrantPermissionRule mRuntimePermissionRuleBluetoothAdmin = GrantPermissionRule.grant(android.Manifest.permission.BLUETOOTH_ADMIN);

    List<DiscoveredPrinterInformation> foundPrinters = new ArrayList<>();
    PrinterDiscovery printerDiscovery;
    private final String PRINTER_NAME = "M611-142";
    Context context;

    @Before
    public void bluetoothPrinterDiscoverySetup(){
        foundPrinters = new ArrayList<>();
        context = androidx.test.core.app.ApplicationProvider.getApplicationContext();
        List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
        printerDiscoveryListeners.add(this);
        printerDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners);
    }

    @Test
    public void test_connectToBTPrinterNotConnectedViaPhoneSettings() {

        // Note: be sure M611-142 printer is turned on nearby but NOT connected (tap 'forget device' on phone's bt settings page)

        try{

            DiscoveredPrinterInformation printer = discoverPrinter(PRINTER_NAME);

            List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
            printerDiscoveryListeners.add(this);
            PrinterDetails printerDetails = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners).connectToDiscoveredPrinter(context, printer, null);

            // be sure to manually tap 'Pair' on phone popup and 'Confirm' on printer screen!!

            assertNotNull(printerDetails);

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    @Test
    public void test_connectToBTPrinterConnectedViaPhoneSettings() {

        // Note: be sure M611-142 printer is turned on nearby and connected via phone settings

        try{

            DiscoveredPrinterInformation printer = discoverPrinter(PRINTER_NAME);

            List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
            printerDiscoveryListeners.add(this);
            PrinterDetails printerDetails = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners).connectToDiscoveredPrinter(context, printer, null);

            assertNotNull(printerDetails);

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    private DiscoveredPrinterInformation discoverPrinter(String printerName){
        try{
            printerDiscovery.startBluetoothPrinterDiscovery(false);

            // wait for the printer to be found
            Thread.sleep(30000);

            for(DiscoveredPrinterInformation printer : foundPrinters) {
                if(printer.getName().equals(printerName)) {
                    return printer;
                }
            }

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }

       return null;
    }

    //region overrides of PrinterDiscoveryListener

    @Override
    public void printerDiscovered(DiscoveredPrinterInformation discoveredPrinterInformation) {

        foundPrinters.add(discoveredPrinterInformation);
    }

    @Override
    public void printerDiscoveryStarted() {
    }

    @Override
    public void printerDiscoveryStopped() {
    }

    //endregion overrides of PrinterDiscoveryListener
}
