package com.example.bradysdk.printercommunication;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import android.Manifest;
import android.content.Context;

import androidx.test.rule.GrantPermissionRule;

import com.bradysdk.api.printerconnection.PrinterDetails;
import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

public class WifiPrinterConnectionTests implements PrinterDiscoveryListener {

    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation = GrantPermissionRule.grant(android.Manifest.permission.ACCESS_FINE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation2 = GrantPermissionRule.grant(Manifest.permission.ACCESS_COARSE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation3 = GrantPermissionRule.grant(Manifest.permission.ACCESS_BACKGROUND_LOCATION);

    @Rule public GrantPermissionRule mRuntimePermissionRuleInternet = GrantPermissionRule.grant(android.Manifest.permission.INTERNET);
    @Rule public GrantPermissionRule mRuntimePermissionRuleInternetMultiCast = GrantPermissionRule.grant(android.Manifest.permission.CHANGE_WIFI_MULTICAST_STATE);

    List<DiscoveredPrinterInformation> foundPrinters = new ArrayList<>();
    PrinterDiscovery printerDiscovery;
    private final String PRINTER_NAME = "M611-142";
    Context context;

    @Before
    public void wifiPrinterDiscoverySetup(){
        foundPrinters = new ArrayList<>();
        context = androidx.test.core.app.ApplicationProvider.getApplicationContext();
        List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
        printerDiscoveryListeners.add(this);
        printerDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners);
    }

    @After
    public void wifiPrinterDiscoveryTeardown(){
        printerDiscovery.stopPrinterDiscovery();
    }

    @Test
    public void test_connectToWifPrinter() {

        // Note: be sure M611-142 printer is turned on nearby

        try{

            DiscoveredPrinterInformation printer = discoverPrinter(PRINTER_NAME);

            List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
            printerDiscoveryListeners.add(this);
            PrinterDetails printerDetails = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners).connectToDiscoveredPrinter(context, printer, null);

            assertNotNull(printerDetails);

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }



    private DiscoveredPrinterInformation discoverPrinter(String printerName){
        try{
            printerDiscovery.startWifiPrinterDiscovery();

            // wait for the printer to be found
            Thread.sleep(30000);

            for(DiscoveredPrinterInformation printer : foundPrinters) {
                if(printer.getName().equals(printerName)) {
                    return printer;
                }
            }

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }

        return null;
    }

    //region overrides of PrinterDiscoveryListener

    @Override
    public void printerDiscovered(DiscoveredPrinterInformation discoveredPrinterInformation) {

        foundPrinters.add(discoveredPrinterInformation);
    }

    @Override
    public void printerDiscoveryStarted() {
    }

    @Override
    public void printerDiscoveryStopped() {
    }

    //endregion overrides of PrinterDiscoveryListener
}
