package com.example.bradysdk.printercommunication;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import android.Manifest;
import android.content.Context;

import androidx.test.rule.GrantPermissionRule;

import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;


public class WifiPrinterDiscoveryTests implements PrinterDiscoveryListener {

    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation = GrantPermissionRule.grant(android.Manifest.permission.ACCESS_FINE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation2 = GrantPermissionRule.grant(Manifest.permission.ACCESS_COARSE_LOCATION);
    @Rule public GrantPermissionRule mRuntimePermissionRuleLocation3 = GrantPermissionRule.grant(Manifest.permission.ACCESS_BACKGROUND_LOCATION);

    @Rule public GrantPermissionRule mRuntimePermissionRuleInternet = GrantPermissionRule.grant(android.Manifest.permission.INTERNET);
    @Rule public GrantPermissionRule mRuntimePermissionRuleInternetMultiCast = GrantPermissionRule.grant(android.Manifest.permission.CHANGE_WIFI_MULTICAST_STATE);

    List<DiscoveredPrinterInformation> foundPrinters = new ArrayList<>();
    PrinterDiscovery printerDiscovery;
    private final String PRINTER_NAME = "M611-142";


    @Before
    public void wifiPrinterDiscoverySetup(){
        foundPrinters = new ArrayList<>();
        Context context = androidx.test.core.app.ApplicationProvider.getApplicationContext();
        List<PrinterDiscoveryListener> printerDiscoveryListeners = new ArrayList<>();
        printerDiscoveryListeners.add(this);
        printerDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(context, printerDiscoveryListeners);
    }

    @After
    public void wifiPrinterDiscoveryTeardown(){
        printerDiscovery.stopPrinterDiscovery();
    }

    @Test
    public void test_findWifiPrintersViaDiscovery() {

        // Note: be sure at least one M611 printer is turned on nearby but NOT connected

        try{
            printerDiscovery.startWifiPrinterDiscovery();

            // wait for the printer to be found
            Thread.sleep(30000);

            assertTrue(foundPrinters.size() > 0);

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }

    @Test
    public void test_findSpecificWifiPrinterViaDiscovery() {

        // Note: be sure M611-142 printer is turned on nearby but NOT connected

        try{
            printerDiscovery.startWifiPrinterDiscovery();

            // wait for the printer to be found
            Thread.sleep(30000);

            boolean m11Production3Found = false;
            for(DiscoveredPrinterInformation printer : foundPrinters) {
                if(printer.getName().equals(PRINTER_NAME)) {
                    m11Production3Found = true;
                    break;
                }
            }
            assertTrue(m11Production3Found);

        } catch (Exception e) {
            fail(e.getMessage());
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }





    //region overrides of PrinterDiscoveryListener

    @Override
    public void printerDiscovered(DiscoveredPrinterInformation discoveredPrinterInformation) {

        foundPrinters.add(discoveredPrinterInformation);
    }

    @Override
    public void printerDiscoveryStarted() {
    }

    @Override
    public void printerDiscoveryStopped() {
    }

    //endregion overrides of PrinterDiscoveryListener

}
