package com.example.bradysdk;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.bradysdk.api.printerconnection.CutOption;
import com.bradysdk.api.printerconnection.PrinterDetails;
import com.bradysdk.api.printerconnection.PrinterProperties;
import com.bradysdk.api.printerconnection.PrinterUpdateListener;
import com.bradysdk.api.printerconnection.PrintingOptions;
import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.api.templates.Template;
import com.bradysdk.api.templates.TemplateObjectData;
import com.bradysdk.api.templates.TemplateObjectType;
import com.bradysdk.printengine.common.Color;
import com.bradysdk.printengine.printerservices.BleGenericPrinterInformation;
import com.bradysdk.printengine.printerservices.BluetoothPrinterInformation;
import com.bradysdk.printengine.printerservices.WifiPrinterInformation;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;
import com.bradysdk.printengine.templateinterface.TemplateFactory;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;

/**
 * The Test Apps front end that handles all views on the home screen
 */
public class MainActivity extends AppCompatActivity implements MainInterface, PrinterUpdateListener {

    private final MainPresenter presenter = new MainPresenter(this, this);
    private EditText messageEditText;
    private Button disconnectButton;
    private Button previewButton;
    private Button printButton;
    private Button cutButton;
    private Button feedButton;
    private Button stopDiscoveryButton;
    private Button startDiscoveryButton;
    private ProgressBar progressBar;
    private ImageView preview;
    private Template template;
    private String templateName;
    private PrinterDetails printerDetails;
    private Spinner templatesSpinner;
    private ListView listView;
    private final List<String> fileList = new ArrayList<>();
    private final List<DiscoveredPrinterInformation> foundPrinters = new ArrayList<>();
    private List<String> foundPrintersNames = new ArrayList<>();
    private PrinterDiscovery discovery;
    private boolean scanStopped = false;
    private boolean permissionGrantedFlag = false;
    private SharedPreferences prefs;
    private Bitmap bitPreview;
    private Bitmap bitmap;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        init();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if(discovery != null)
            discovery.onResume();
    }

    @Override
    protected void onPause(){
        super.onPause();
        if(discovery != null)
            discovery.onPause();
    }

    @Override
    protected void onDestroy(){
        super.onDestroy();
        if(discovery != null)
            discovery.onDestroy();
    }

    /**
     * Initializes all views on the home screen
     */
    public void init(){
        messageEditText = findViewById(R.id.messageEditText);
        disconnectButton = findViewById(R.id.disconnectButton);
        previewButton = findViewById(R.id.previewButton);
        printButton = findViewById(R.id.printButton);
        cutButton = findViewById(R.id.cutLabelButton);
        feedButton = findViewById(R.id.feedButton);
        stopDiscoveryButton = findViewById(R.id.stopDiscoveryButton);
        startDiscoveryButton = findViewById(R.id.startDiscoveryButton);
        progressBar = findViewById(R.id.progressBar);
        preview = findViewById(R.id.previewBitmap);
        templatesSpinner = findViewById(R.id.templatesSpinner);
        listView = findViewById(R.id.printersListView);

        prefs = getSharedPreferences("BRADY_ANDROID_SDK", MODE_PRIVATE);
        if(prefs.getBoolean("FIRST_RUN", true)) {
            prefs.edit().putBoolean("FIRST_RUN", false).apply();
            requestAllPermissions();
        }
        presenter.setupPrinterDiscovery();
        initializeSpinner();
        initializeListView();
        messageEditText.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(bitPreview != null) {
                    bitPreview.recycle();
                }
                bitPreview = null;
                preview.setImageBitmap(bitPreview);
            }
        });

        messageEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if(bitPreview != null) {
                    bitPreview.recycle();
                }
                bitPreview = null;
                preview.setImageBitmap(bitPreview);
            }
        });
    }

    private void requestAllPermissions() {

        if (ContextCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_CONNECT) != PackageManager.PERMISSION_GRANTED ||
                ContextCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH_SCAN) != PackageManager.PERMISSION_GRANTED ||
                ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED ||
                ContextCompat.checkSelfPermission(this, Manifest.permission.BLUETOOTH) != PackageManager.PERMISSION_GRANTED) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.BLUETOOTH_CONNECT, Manifest.permission.BLUETOOTH_SCAN,
                        Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.BLUETOOTH}, 0);
            }
            else{
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.BLUETOOTH}, 0);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if(requestCode == 0){
            if(grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED){
                startDiscoveryButton.setVisibility(View.VISIBLE);
                System.out.println("PERMISSION GRANTED");
            }
        }
    }

    /**
     * Populates the spinner with all internal templates
     */
    private void initializeSpinner(){
        getAllTemplateNames();
        ArrayAdapter<String> adapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, fileList);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        templatesSpinner.setAdapter(adapter);

        templatesSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
                String selectedFileName = templatesSpinner.getSelectedItem().toString();
                InputStream iStream = getResources().openRawResource(getResources().getIdentifier(selectedFileName.split("[.]")[0], "raw", getPackageName()));
                if(selectedFileName.endsWith(".BWS") || selectedFileName.endsWith(".BWT")) {
                    template = TemplateFactory.getTemplate(iStream, MainActivity.this);

                    for (TemplateObjectData placeholder : template.getTemplateData()) {
                        messageEditText.setText(placeholder.getValue());
                    }
                }
                else {
                    bitmap = BitmapFactory.decodeStream(iStream);
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {

            }
        });
    }

    /**
     * Initializes the List View so that discovered printers can be seen by the user
     */
    private void initializeListView() {
        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                System.out.println("LIST ITEM SELECTED: " + i);
                for(DiscoveredPrinterInformation info : foundPrinters){
                    String listViewItem = listView.getItemAtPosition(i).toString();
                    String printerType = listView.getItemAtPosition(i).toString().split(" ")[1];
                    if(listViewItem.contains(info.getName())){
                        if(printerType.equals("(BLE)") && info instanceof BleGenericPrinterInformation) {
                            presenter.connectToPrinter(info, MainActivity.this, true);
                        }
                        else if(printerType.equals("(Bluetooth") && info instanceof BluetoothPrinterInformation) {
                            presenter.connectToPrinter(info, MainActivity.this, true);
                        }
                        else if(printerType.equals("(Wifi)") && info instanceof WifiPrinterInformation) {
                            presenter.connectToPrinter(info, MainActivity.this, true);
                        }
                    }
                }

            }
        });

        startDiscoveryScan();
    }

    private void startDiscoveryScan() {
        Thread updateListView = new Thread() {
            @Override
            public void run() {
                while (!scanStopped) {
                    getFoundPrinters();
                    try {
                        Thread.sleep(5000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        };
        updateListView.start();
    }

    /**
     * Grabs all internal template names for the spinner
     */
    private void getAllTemplateNames(){
        for(Field field : R.raw.class.getFields()) {
            presenter.getTemplate(getResources().getIdentifier(field.getName(), "raw", getPackageName()));
            fileList.add(templateName);
        }
    }

    /**
     * Shows the user a preview of what will print
     * @param view Preview Button on click
     */
    public void previewTemplate(View view) {
        try {
            if(bitmap == null) {

                // Only for debug : template.getTemplateData()[0].setValue(messageEditText.getText().toString());
                Hashtable<String, String> editTextsAsDictionary = new Hashtable<>();

                for (TemplateObjectData placeholder : template.getTemplateData()) {
                    if (placeholder.getTemplateObjectType() == TemplateObjectType.Text) {
                        editTextsAsDictionary.put(placeholder.getName(), messageEditText.getText().toString());
                        placeholder.setValue(editTextsAsDictionary.get(placeholder.getName()));
                    }
                }

                //"preview" is the ImageView object in our Activity.
                double elementSize = Math.min(preview.getWidth(), preview.getHeight());
                bitPreview = template.getPreview(0, 96, elementSize);
                preview.setImageBitmap(bitPreview);
            }
            else {
                preview.setImageBitmap(bitmap);
            }

            printButton.setEnabled(true);
        }
        catch (Exception e) { System.out.println("Field Cannot Be Empty!");}
    }

    /**
     * Tells the presenter to attempt to connect to the selected printer
     * @param view Connect Button on Click
     */
    public void disconnectFromPrinter(View view) {
        if(printerDetails != null) {
            printerDetails.disconnect();
            try {
                runOnUiThread(() -> {
                    disconnectButton.setVisibility(View.INVISIBLE);
                    feedButton.setEnabled(false);
                    previewButton.setEnabled(false);
                    cutButton.setEnabled(false);
                    printButton.setEnabled(false);
                    preview.setImageBitmap(null);
                });

                presenter.setupPrinterDiscovery();
                scanStopped = false;
                startDiscoveryScan();
            }
            catch (Exception e) {
            }
        }
    }

    /**
     * Tells the presenter to attempt to print the desired template
     * @param view Print Button on Click
     */
    public void printToPrinter(View view) {
        if(bitmap == null)
            presenter.print(template);
        else
            presenter.print(bitmap);
    }

    public void stopDiscovery(View view){
        presenter.stopDiscovery();
        scanStopped = true;
    }

    @Override
    public void PrinterUpdate(List<PrinterProperties> list) {
        for(PrinterProperties property : list){
            if(property == PrinterProperties.CurrentStatus) {

            }
        }
    }

    @Override
    public void getTemplate(String templateName) {
        this.templateName = templateName;
    }

    /**
     * After successfully connecting to the desired printer, setPrinterDetails sets all of the
     * printer details in order to populate the UI
     * @param printerDetails connected printers details
     * @param showDetailPage boolean that determines if the details page is shown to the user
     */
    @Override
    public void setPrinterDetails(PrinterDetails printerDetails, boolean showDetailPage) {
        this.printerDetails = printerDetails;

        try {
            Intent intent = new Intent(this, PrinterActivity.class);
            intent.putExtra("PRINTER_MODEL", printerDetails.getPrinterModel());
            intent.putExtra("PRINTER_NAME", printerDetails.getPrinterName());
            intent.putExtra("BATTERY_LEVEL", Integer.toString(printerDetails.getBatteryLevelPercentage()));
            intent.putExtra("PRINTER_STATUS", printerDetails.getPrinterStatus().name() + " (" + printerDetails.getConnectionType() + ")");
            intent.putExtra("PRINTER_SUPPLY", printerDetails.getSupplyName());
            intent.putExtra("Y_NUMBER", printerDetails.getYNumber());
            intent.putExtra("PRINTER_SUPPLY_LEFT", Integer.toString(printerDetails.getSupplyRemainingPercentage()));
            intent.putExtra("SUPPLY_DIMENSIONS", printerDetails.getSupplyDimensions());
            intent.putExtra("PRINTER_RIBBON", printerDetails.getRibbonName());
            intent.putExtra("PRINTER_RIBBON_LEFT", Integer.toString(printerDetails.getRibbonRemainingPercentage()));

            runOnUiThread(()->{
                previewButton.setEnabled(true);
                if(printerDetails.getPrinterModel().equals("M211") || printerDetails.getPrinterModel().equals("M511")) {
                    cutButton.setEnabled(true);
                    feedButton.setEnabled(true);
                }
                disconnectButton.setVisibility(View.VISIBLE);
                disconnectButton.setEnabled(true);
                stopDiscoveryButton.setEnabled(true);
                disconnectButton.setBackgroundColor(Color.GREEN);
            });

            if(showDetailPage) {
                startActivity(intent);
            }
        }
        catch(Exception e){
            System.out.println("PRINTER DETAILS ARE NOT COMPLETE!");
            runOnUiThread(()->disconnectButton.setBackgroundColor(Color.RED));
            if(presenter.getDiscoveredPrinterNames().isEmpty()) {
                runOnUiThread(() -> startDiscoveryButton.setVisibility(View.VISIBLE));
            }
        }
    }

    /**
     * Shows loading wheel
     */
    @Override
    public void showProgress() {
        runOnUiThread(()->progressBar.setVisibility(View.VISIBLE));
    }

    /**
     * Hides loading wheel
     */
    @Override
    public void hideProgress() {
        runOnUiThread(()->progressBar.setVisibility(View.INVISIBLE));
    }

    @Override
    public void sendPrintDiscovery(PrinterDiscovery printerDiscovery) {
        this.discovery = printerDiscovery;
    }

    public void showToast(final String toast){
        runOnUiThread(()-> Toast.makeText(this, toast, Toast.LENGTH_LONG).show());
    }

    public void cutLabel(View view) {
        presenter.cutLabel();
    }

    public void feedLabel(View view) {
        presenter.feed();
    }

    public void startDiscovery(View view){
        presenter.setupPrinterDiscovery();
        startDiscoveryButton.setVisibility(View.INVISIBLE);
    }

    public void getFoundPrinters() {

        for(DiscoveredPrinterInformation printer : presenter.getDiscoveredPrinterNames()) {
            if(printer instanceof BleGenericPrinterInformation) {
                if(!foundPrintersNames.contains(printer.getName() + " (BLE)")) {
                    foundPrintersNames.add(printer.getName() + " (BLE)");
                    foundPrinters.add(printer);
                }
            }
            else if(printer instanceof BluetoothPrinterInformation) {
                if(!foundPrintersNames.contains(printer.getName() + " (Bluetooth)")) {
                    foundPrintersNames.add(printer.getName() + " (Bluetooth)");
                    foundPrinters.add(printer);
                }
            }
            else if(printer instanceof WifiPrinterInformation){
                if(!foundPrintersNames.contains(printer.getName() + " (Wifi)")) {
                    foundPrintersNames.add(printer.getName() + " (Wifi)");
                    foundPrinters.add(printer);
                }
            }
        }

        if(foundPrintersNames != null) {
            ArrayAdapter<String> printersListAdapter = new ArrayAdapter<String>
                    (this, android.R.layout.simple_list_item_1, foundPrintersNames) {
                @Override
                public View getView(int position, View convertView, ViewGroup parent) {
                    View view = super.getView(position, convertView, parent);
                    TextView textView = view.findViewById(android.R.id.text1);
                    textView.setTextColor(Color.GREEN);
                    return view;
                }
            };
            runOnUiThread(() -> listView.setAdapter(printersListAdapter));
        }
    }
}